@@ -16,12 +16,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from util import *
-from bitcoin import *
+import datetime
+import time
+import copy
+from util import print_msg, format_satoshis, print_stderr
+from bitcoin import is_valid, hash_160_to_bc_address, hash_160
 from decimal import Decimal
 import bitcoin
 from transaction import Transaction
 
+
 class Command:
     def __init__(self, name, min_args, max_args, requires_network, requires_wallet, requires_password, description, syntax = '', options_syntax = ''):
         self.name = name
@@ -34,7 +38,10 @@ class Command:
         self.syntax = syntax
         self.options = options_syntax
 
+
 known_commands = {}
+
+
 def register_command(*args):
     global known_commands
     name = args[0]
@@ -51,7 +58,6 @@ paytomany_syntax = "paytomany <recipient> <amount> [<recipient> <amount> ...]\n<
 signmessage_syntax = 'signmessage <address> <message>\nIf you want to lead or end a message with spaces, or want double spaces inside the message make sure you quote the string. I.e. " Hello  This is a weird String "'
 verifymessage_syntax = 'verifymessage <address> <signature> <message>\nIf you want to lead or end a message with spaces, or want double spaces inside the message make sure you quote the string. I.e. " Hello  This is a weird String "'
 
-
 #                command
 #                                              requires_network
 #                                                     requires_wallet
@@ -59,43 +65,53 @@ verifymessage_syntax = 'verifymessage <address> <signature> <message>\nIf you wa
 register_command('contacts',             0, 0, False, True,  False, 'Show your list of contacts')
 register_command('create',               0, 0, False, True,  False, 'Create a new wallet')
 register_command('createmultisig',       2, 2, False, True,  False, 'similar to bitcoind\'s command')
-register_command('createrawtransaction', 2, 2, False, True,  False, 'similar to bitcoind\'s command')
+register_command('createrawtransaction', 2, 2, False, True,  False, 'Create an unsigned transaction. The syntax is similar to bitcoind.')
 register_command('deseed',               0, 0, False, True,  False, 'Remove seed from wallet, creating a seedless, watching-only wallet.')
 register_command('decoderawtransaction', 1, 1, False, False, False, 'similar to bitcoind\'s command')
-register_command('dumpprivkey',          1, 1, False, True,  True,  'Dumps a specified private key for a given address', 'dumpprivkey <bitcoin address>')
-register_command('dumpprivkeys',         0, 0, False, True,  True,  'dump all private keys')
+register_command('getprivatekeys',       1, 1, False, True,  True,  'Get the private keys of a given address', 'getprivatekeys <bitcoin address>')
+register_command('dumpprivkeys',         0, 0, False, True,  True,  'Dump all private keys in your wallet')
 register_command('freeze',               1, 1, False, True,  True,  'Freeze the funds at one of your wallet\'s addresses', 'freeze <address>')
 register_command('getbalance',           0, 1, True,  True,  False, 'Return the balance of your wallet, or of one account in your wallet', 'getbalance [<account>]')
 register_command('getservers',           0, 0, True,  False, False, 'Return the list of available servers')
-register_command('getversion',           1, 1, False,  False,  False, 'Return the version of your client', 'getversion')
-register_command('getaddressbalance',    1, 1, True,  True,  False, 'Return the balance of an address', 'getbalance <address>')
-register_command('getaddresshistory',    1, 1, True,  True,  False, 'Return the transaction history of a wallet address', 'getaddresshistory <address>')
+register_command('getversion',           0, 0, False, False, False, 'Return the version of your client', 'getversion')
+register_command('getaddressbalance',    1, 1, True,  False, False, 'Return the balance of an address', 'getaddressbalance <address>')
+register_command('getaddresshistory',    1, 1, True,  False, False, 'Return the transaction history of a wallet address', 'getaddresshistory <address>')
 register_command('getconfig',            1, 1, False, False, False, 'Return a configuration variable', 'getconfig <name>')
 register_command('getpubkeys',           1, 1, False, True,  False, 'Return the public keys for a wallet address', 'getpubkeys <bitcoin address>')
-register_command('getrawtransaction',    1, 2, True,  False, False, 'Retrieve a transaction', 'getrawtransaction <txhash> <height>')
+register_command('getrawtransaction',    1, 1, True,  False, False, 'Retrieve a transaction', 'getrawtransaction <txhash>')
 register_command('getseed',              0, 0, False, True,  True,  'Print the generation seed of your wallet.')
 register_command('getmpk',               0, 0, False, True,  False, 'Return your wallet\'s master public key', 'getmpk')
 register_command('help',                 0, 1, False, False, False, 'Prints this help')
 register_command('history',              0, 0, True,  True,  False, 'Returns the transaction history of your wallet')
 register_command('importprivkey',        1, 1, False, True,  True,  'Import a private key', 'importprivkey <privatekey>')
+register_command('ismine',               1, 1, False, True,  False, 'Return true if and only if address is in wallet', 'ismine <address>')
 register_command('listaddresses',        2, 2, False, True,  False, 'Returns your list of addresses.', '', listaddr_options)
 register_command('listunspent',          0, 0, True,  True,  False, 'Returns the list of unspent inputs in your wallet.')
+register_command('getaddressunspent',    1, 1, True,  False, False, 'Returns the list of unspent inputs for an address.')
 register_command('mktx',                 5, 5, False, True,  True,  'Create a signed transaction', 'mktx <recipient> <amount> [label]', payto_options)
 register_command('mksendmanytx',         4, 4, False, True,  True,  'Create a signed transaction', mksendmany_syntax, payto_options)
 register_command('payto',                5, 5, True,  True,  True,  'Create and broadcast a transaction.', payto_syntax, payto_options)
 register_command('paytomany',            4, 4, True,  True,  True,  'Create and broadcast a transaction.', paytomany_syntax, payto_options)
 register_command('password',             0, 0, False, True,  True,  'Change your password')
 register_command('restore',              0, 0, True,  True,  False, 'Restore a wallet', '', restore_options)
+register_command('searchcontacts',       1, 1, False, True,  False,  'Search through contacts, return matching entries', 'searchcontacts <query>')
 register_command('setconfig',            2, 2, False, False, False, 'Set a configuration variable', 'setconfig <name> <value>')
 register_command('setlabel',             2,-1, False, True,  False, 'Assign a label to an item', 'setlabel <tx_hash> <label>')
 register_command('sendrawtransaction',   1, 1, True,  False, False, 'Broadcasts a transaction to the network.', 'sendrawtransaction <tx in hexadecimal>')
-register_command('signrawtransaction',   1, 3, False, True,  True,  'similar to bitcoind\'s command')
+register_command('signtxwithkey',        1, 3, False, False, False, 'Sign a serialized transaction with a key','signtxwithkey <tx> <key>')
+register_command('signtxwithwallet',     1, 3, False, True,  True,  'Sign a serialized transaction with a wallet','signtxwithwallet <tx>')
 register_command('signmessage',          2,-1, False, True,  True,  'Sign a message with a key', signmessage_syntax)
 register_command('unfreeze',             1, 1, False, True,  False, 'Unfreeze the funds at one of your wallet\'s address', 'unfreeze <address>')
 register_command('validateaddress',      1, 1, False, False, False, 'Check that the address is valid', 'validateaddress <address>')
 register_command('verifymessage',        3,-1, False, False, False, 'Verifies a signature', verifymessage_syntax)
 
-
+register_command('encrypt',              2,-1, False, False, False, 'encrypt a message with pubkey','encrypt <pubkey> <message>')
+register_command('decrypt',              2,-1, False, True, True,   'decrypt a message encrypted with pubkey','decrypt <pubkey> <message>')
+register_command('getproof',             1, 1, True, False, False, 'get merkle proof', 'getproof <address>')
+register_command('getutxoaddress',       2, 2, True, False, False, 'get the address of an unspent transaction output','getutxoaddress <txid> <pos>')
+register_command('sweep',                2, 3, True, False, False, 'Sweep a private key.', 'sweep privkey addr [fee]')
+register_command('make_seed',            3, 3, False, False, False, 'Create a seed.','options: --nbits --entropy --lang')
+register_command('check_seed',           1,-1, False, False, False, 'Check that a seed was generated with external entropy. Option: --entropy --lang')
 
 
 class Commands:
@@ -117,56 +133,85 @@ class Commands:
             apply(self._callback, ())
         return result
 
+    def make_seed(self, nbits, custom_entropy, language):
+        from mnemonic import Mnemonic
+        s = Mnemonic(language).make_seed(nbits, custom_entropy=custom_entropy)
+        return s.encode('utf8')
+
+    def check_seed(self, seed, custom_entropy, language):
+        from mnemonic import Mnemonic
+        return Mnemonic(language).check_seed(seed, custom_entropy)
+
     def getaddresshistory(self, addr):
-        assert self.wallet.is_mine(addr)
-        h = self.wallet.get_history(addr)
-        if h is None: h = self.network.synchronous_get([ ('blockchain.address.get_history',[addr]) ])[0]
-        return h
+        return self.network.synchronous_get([ ('blockchain.address.get_history',[addr]) ])[0]
 
     def listunspent(self):
-        import copy
         l = copy.deepcopy(self.wallet.get_unspent_coins())
         for i in l: i["value"] = str(Decimal(i["value"])/100000000)
         return l
 
+    def getaddressunspent(self, addr):
+        return self.network.synchronous_get([ ('blockchain.address.listunspent',[addr]) ])[0]
+
+    def getutxoaddress(self, txid, num):
+        r = self.network.synchronous_get([ ('blockchain.utxo.get_address',[txid, num]) ])
+        if r:
+            return {'address':r[0] }
+
     def createrawtransaction(self, inputs, outputs):
-        # convert to own format
+        coins = self.wallet.get_unspent_coins(None)
+        tx_inputs = []
         for i in inputs:
-            i['tx_hash'] = i['txid']
-            i['index'] = i['vout']
-        outputs = map(lambda x: (x[0],int(1e8*x[1])), outputs.items())
-        tx = Transaction.from_io(inputs, outputs)
+            prevout_hash = i['txid']
+            prevout_n = i['vout']
+            for c in coins:
+                if c['prevout_hash'] == prevout_hash and c['prevout_n'] == prevout_n:
+                    self.wallet.add_input_info(c)
+                    tx_inputs.append(c)
+                    break
+            else:
+                raise BaseException('Transaction output not in wallet', prevout_hash+":%d"%prevout_n)
+        outputs = map(lambda x: ('address', x[0], int(1e8*x[1])), outputs.items())
+        tx = Transaction(tx_inputs, outputs)
         return tx
 
-    def signrawtransaction(self, raw_tx, input_info, private_keys):
-        tx = Transaction(raw_tx)
-        self.wallet.signrawtransaction(tx, input_info, private_keys, self.password)
+    def signtxwithkey(self, raw_tx, sec):
+        tx = Transaction.deserialize(raw_tx)
+        pubkey = bitcoin.public_key_from_private_key(sec)
+        tx.sign({ pubkey:sec })
+        return tx
+
+    def signtxwithwallet(self, raw_tx):
+        tx = Transaction.deserialize(raw_tx)
+        self.wallet.sign_transaction(tx, self.password)
         return tx
 
     def decoderawtransaction(self, raw):
-        tx = Transaction(raw)
-        return tx.deserialize()
+        tx = Transaction.deserialize(raw)
+        return {'inputs':tx.inputs, 'outputs':tx.outputs}
 
     def sendrawtransaction(self, raw):
-        tx = Transaction(raw)
-        r, h = self.wallet.sendtx( tx )
-        return h
+        tx = Transaction.deserialize(raw)
+        return self.network.synchronous_get([('blockchain.transaction.broadcast', [str(tx)])])[0]
 
     def createmultisig(self, num, pubkeys):
         assert isinstance(pubkeys, list)
         redeem_script = Transaction.multisig_script(pubkeys, num)
         address = hash_160_to_bc_address(hash_160(redeem_script.decode('hex')), 5)
         return {'address':address, 'redeemScript':redeem_script}
-    
+
     def freeze(self,addr):
         return self.wallet.freeze(addr)
-        
+
     def unfreeze(self,addr):
         return self.wallet.unfreeze(addr)
 
-    def dumpprivkey(self, addr):
+    def getprivatekeys(self, addr):
         return self.wallet.get_private_key(addr, self.password)
 
+    def ismine(self, addr):
+        return self.wallet.is_mine(addr)
+
     def dumpprivkeys(self, addresses = None):
         if addresses is None:
             addresses = self.wallet.addresses(True)
@@ -180,13 +225,8 @@ class Commands:
         return out
 
     def getpubkeys(self, addr):
-        assert is_valid(addr) and self.wallet.is_mine(addr)
         out = { 'address':addr }
-        account, sequence = self.wallet.get_address_index(addr)
-        if account != -1:
-            a = self.wallet.accounts[account]
-            out['pubkeys'] = a.get_pubkeys( sequence )
-
+        out['pubkeys'] = self.wallet.get_public_keys(addr)
         return out
 
     def getbalance(self, account= None):
@@ -200,25 +240,33 @@ class Commands:
         return out
 
     def getaddressbalance(self, addr):
-        c, u = self.wallet.get_addr_balance(addr)
-        out = { "confirmed": str(Decimal(c)/100000000) }
-        if u: out["unconfirmed"] = str(Decimal(u)/100000000)
+        out = self.network.synchronous_get([ ('blockchain.address.get_balance',[addr]) ])[0]
+        out["confirmed"] =  str(Decimal(out["confirmed"])/100000000)
+        out["unconfirmed"] =  str(Decimal(out["unconfirmed"])/100000000)
+        return out
+
+    def getproof(self, addr):
+        p = self.network.synchronous_get([ ('blockchain.address.get_proof',[addr]) ])[0]
+        out = []
+        for i,s in p:
+            out.append(i)
         return out
 
     def getservers(self):
+        while not self.network.is_up_to_date():
+            time.sleep(0.1)
         return self.network.get_servers()
 
     def getversion(self):
-        import electrum 
+        import electrum  # Needs to stay here to prevent ciruclar imports
         return electrum.ELECTRUM_VERSION
- 
+
     def getmpk(self):
-        return self.wallet.get_master_public_key()
+        return self.wallet.get_master_public_keys()
 
     def getseed(self):
-        mnemonic = self.wallet.get_mnemonic(self.password)
-        seed = self.wallet.get_seed(self.password)
-        return { 'mnemonic':mnemonic, 'seed':seed, 'version':self.wallet.seed_version }
+        s = self.wallet.get_mnemonic(self.password)
+        return s.encode('utf8')
 
     def importprivkey(self, sec):
         try:
@@ -228,17 +276,17 @@ class Commands:
             out = "Error: Keypair import failed: " + str(e)
         return out
 
+    def sweep(self, privkey, to_address, fee = 0.0001):
+        fee = int(Decimal(fee)*100000000)
+        return Transaction.sweep([privkey], self.network, to_address, fee)
 
     def signmessage(self, address, message):
         return self.wallet.sign_message(address, message, self.password)
 
-
     def verifymessage(self, address, signature, message):
         return bitcoin.verify_message(address, signature, message)
 
-
     def _mktx(self, outputs, fee = None, change_addr = None, domain = None):
-
         for to_address, amount in outputs:
             if not is_valid(to_address):
                 raise Exception("Invalid Bitcoin address", to_address)
@@ -251,7 +299,7 @@ class Commands:
             for addr in domain:
                 if not is_valid(addr):
                     raise Exception("invalid Bitcoin address", addr)
-            
+
                 if not self.wallet.is_mine(addr):
                     raise Exception("address not in wallet", addr)
 
@@ -268,12 +316,11 @@ class Commands:
                     break
 
             amount = int(100000000*amount)
-            final_outputs.append((to_address, amount))
-            
+            final_outputs.append(('address', to_address, amount))
+
         if fee: fee = int(100000000*fee)
         return self.wallet.mktx(final_outputs, self.password, fee , change_addr, domain)
 
-
     def mktx(self, to_address, amount, fee = None, change_addr = None, domain = None):
         tx = self._mktx([(to_address, amount)], fee, change_addr, domain)
         return tx
@@ -282,7 +329,6 @@ class Commands:
         tx = self._mktx(outputs, fee, change_addr, domain)
         return tx
 
-
     def payto(self, to_address, amount, fee = None, change_addr = None, domain = None):
         tx = self._mktx([(to_address, amount)], fee, change_addr, domain)
         r, h = self.wallet.sendtx( tx )
@@ -293,9 +339,7 @@ class Commands:
         r, h = self.wallet.sendtx( tx )
         return h
 
-
     def history(self):
-        import datetime
         balance = 0
         out = []
         for item in self.wallet.get_tx_history():
@@ -306,25 +350,25 @@ class Commands:
                 time_str = "----"
 
             label, is_default_label = self.wallet.get_label(tx_hash)
-            if not label: label = tx_hash
-            else: label = label + ' '*(64 - len(label) )
 
-            out.append( "%16s"%time_str + "  " + label + "  " + format_satoshis(value)+ "  "+ format_satoshis(balance) )
+            out.append({'txid':tx_hash, 'date':"%16s"%time_str, 'label':label, 'value':format_satoshis(value), 'confirmations':conf})
         return out
 
-
-
     def setlabel(self, key, label):
         self.wallet.set_label(key, label)
 
-            
-
     def contacts(self):
         c = {}
         for addr in self.wallet.addressbook:
             c[addr] = self.wallet.labels.get(addr)
         return c
 
+    def searchcontacts(self, query):
+        results = {}
+        for addr in self.wallet.addressbook:
+            if query.lower() in self.wallet.labels.get(addr).lower():
+                results[addr] = self.wallet.labels.get(addr)
+        return results
 
     def listaddresses(self, show_all = False, show_label = False):
         out = []
@@ -340,7 +384,7 @@ class Commands:
                     item = addr
                 out.append( item )
         return out
-                         
+
     def help(self, cmd=None):
         if cmd not in known_commands:
             print_msg("\nList of commands:", ', '.join(sorted(known_commands)))
@@ -351,11 +395,20 @@ class Commands:
             if cmd.options: print_msg("options:\n" + cmd.options)
         return None
 
-    def getrawtransaction(self, tx_hash, height = 0):
-        tx = self.wallet.transactions.get(tx_hash)
-        if tx:
-            return tx
-        height = int(height)
-        return self.network.retrieve_transaction(tx_hash, height)
+    def getrawtransaction(self, tx_hash):
+        if self.wallet:
+            tx = self.wallet.transactions.get(tx_hash)
+            if tx:
+                return tx
 
+        raw = self.network.synchronous_get([ ('blockchain.transaction.get',[tx_hash]) ])[0]
+        if raw:
+            return Transaction.deserialize(raw)
+        else:
+            return "unknown transaction"
 
+    def encrypt(self, pubkey, message):
+        return bitcoin.encrypt_message(message, pubkey)
+
+    def decrypt(self, pubkey, message):
+        return self.wallet.decrypt_message(pubkey, message, self.password)
